import getpass

import boto3

from django_iam_dbauth.utils import resolve_cname


def get_aws_connection_params(params):
    enabled = params.pop("use_iam_auth", None)
    if enabled:
        region_name = params.pop("region_name", None)
        session = boto3.session.Session()
        rds_client = session.client(service_name="rds", region_name=region_name)

        hostname = params.get("host")
        if hostname:
            hostname = (
                resolve_cname(hostname)
                if params.pop("resolve_cname_enabled", True)
                else hostname
            )
        else:
            hostname = "localhost"

        params["password"] = rds_client.generate_db_auth_token(
            DBHostname=hostname,
            Port=params.get("port", 5432),
            DBUsername=params.get("user") or getpass.getuser(),
        )

    return params



from django.conf import settings
from moto import mock_aws
import boto3
import pytest


def pytest_configure():
    settings.configure(
        HEALTH_CHECKS={},
        MIDDLEWARE_CLASSES=[],
        SECRET_KEY="0123456789",
        INSTALLED_APPS=[
            "django.contrib.admin",
            "django.contrib.contenttypes",
            "django.contrib.auth",
            "django.contrib.sessions",
        ],
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                "LOCATION": "unique-snowflake",
            }
        },
        DATABASES={
            "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": "db.sqlite"}
        },
        MIDDLEWARE=(
            "django.contrib.sessions.middleware.SessionMiddleware",
            "django.contrib.auth.middleware.AuthenticationMiddleware",
        ),
        TEMPLATES=[
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "OPTIONS": {
                    "loaders": ("django.template.loaders.app_directories.Loader",)
                },
            }
        ],
        USE_TZ=True,
        ROOT_URLCONF="test_urls",
    )


@pytest.fixture
def rds_client():
    # Use moto to mock the RDS client
    with mock_aws():
        session = boto3.session.Session()
        client = session.client(service_name="rds", region_name="us-west-2")
        yield client



from django_iam_dbauth.aws.postgresql.base import DatabaseWrapper


def test_get_connection_params(rds_client):
    settings = {
        "NAME": "example",
        "USER": "postgresql",
        "PASSWORD": "secret",
        "PORT": 5432,
        "HOST": "example-cname.labd.nl",
        "ENGINE": "django_iam_dbauth.aws.postgresql",
        "OPTIONS": {
            "use_iam_auth": 1,
            "region_name": "test",
            "resolve_cname_enabled": False,
        },
    }

    db = DatabaseWrapper(settings)
    params = db.get_connection_params()

    assert params["password"].startswith("example-cname.labd.nl")



import dns.name
import dns.rdatatype
import dns.resolver
from django.db.utils import OperationalError
from dns.exception import DNSException


def resolve_cname(hostname):
    """Resolve a CNAME record to the original RDS endpoint.

    This is required for AWS where the hostname of the RDS instance is part of
    the signing request.

    Looking for the endpoint which is of the form cluster-name.accountandregionhash.regionid.rds.amazonaws.com
    To do so, recursively resolve the host name until it's a subdomain of rds.amazonaws.com.
    """
    base_domain = dns.name.from_text("rds.amazonaws.com")
    answer = dns.name.from_text(hostname)
    while not answer.is_subdomain(base_domain):
        try:
            # Replace deprecated `query` with `resolve`
            # There is one and only one answer for a CNAME and its type is CNAME.
            # If the name doesn't exist or exists with a different type, an exception is raised.
            answer = dns.resolver.resolve(answer, dns.rdatatype.CNAME, search=True)[
                0
            ].target
        except DNSException as e:
            # Break when resolution doesn't work.
            # This avoids cryptic authentication failures from RDS.
            raise OperationalError("Failed to resolve hostname to RDS endpoint.") from e

    return answer.to_text().strip(".")
